import re
import json

#Just a comment

def is_slide_not_skipped(slide) -> bool:
    slide_properties = slide.get('slideProperties')
    if 'isSkipped' in slide_properties:
        if slide_properties.get('isSkipped') == True:
            return False
    return True

def is_slide_bios(slide) -> bool:
    if len(slide.get(
            'pageElements')) > 5:  # the number of 5 comes from the assumptions that the bios should have at 5 pages elemenets(Name, Title and AND Title, Overview, Core focus and key skills)
        return True
    return False

def delete_skipped_slides(slides) -> list:
    return list(filter(is_slide_not_skipped, slides))

def delete_slides_no_bios(slides) -> list:
    return list(filter(is_slide_bios, slides))

def preprocess_text(text):
    no_special_chars = remove_special_characters(text)
    no_space_text = remove_consecutive_spaces(no_special_chars)
    return no_space_text

def remove_special_characters(text): #keeps only characters, numbers, dots, comma, new line, slashes
    return re.sub('[^a-zA-Z0-9 \.\,\/\\n\(\)\|]', '', text)
'''
def remove_consecutive_new_lines(text):
    return re.sub('[\\n]+', '\\n', text)
'''

def remove_consecutive_spaces(text):
    return re.sub('[\s+]', ' ', text)

def page_elements_to_dict(slide):
    page_elements = slide.get('pageElements')
    dict = {}
    for count, page_element in enumerate(page_elements):
        try:
            page_element_id = page_element.get('objectId')
            if 'shape' in page_element:
                shape = page_element.get('shape')
                if 'text' in shape:
                    text_elements = page_element.get('shape').get('text').get('textElements')
                    content = ""
                    for text_element in text_elements:
                        if 'textRun' in text_element:
                            text = text_element.get('textRun').get('content') # ignore special characters
                            content += text
                    content = content.strip()
                    if content != "":
                        dict[page_element_id] = preprocess_text(content)
            else:
                continue
        except Exception as e:
            print(e)
    return dict


def convert_bio_slides_to_dict(slides):
    slides_bio_dict = {}
    for slide in slides:
        slides_bio_dict[slide.get('objectId')] = page_elements_to_dict(slide)
    return slides_bio_dict

def preprocess_raw_file(raw_file_path):
    try:
        with open(raw_file_path) as raw_file:
            raw_data = json.load(raw_file)
        slides = raw_data.get('slides')
        slides = delete_skipped_slides(slides)
        slides = delete_slides_no_bios(slides)
        slides = convert_bio_slides_to_dict(slides)
        return slides
    except Exception as e:
        print('Could not preprocess the data due to issue: ' + str(e))
        return None


from sqlalchemy import Boolean, Column, ForeignKey, Integer, String, Text, DateTime
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.dialects.postgresql import TSVECTOR

Base = declarative_base()

class Person(Base):
    __tablename__ = 'Persons'

    id = Column(Integer, unique = True, primary_key = True)
    first_name = Column(String, nullable = False)
    last_name = Column(String, nullable = False)
    age = Column(Integer)
    gender = Column(String)
    nationality = Column(String)
    hasWatch = Column(Boolean)

    def __repr__(self):
        return f'Person: first_name:{self.first_name}'

class Presentation(Base):
    __tablename__ = "google_presentations"

    presentation_id = Column(Integer, primary_key=True, index=True)
    presentation_link = Column(String, unique=True)
    club_name = Column(String, unique=True)
    revision_id = Column(String)
    created_at = Column(DateTime)
    updated_at = Column(DateTime)
    
    slides = relationship('Slide',back_populates = 'presentation')

    def __repr__(self):
        return f'<Presentation club:{self.club_name} presentation_id:{self.presentation_id} link:{self.presentation_link}>'

class Slide(Base):
    __tablename__ = 'google_slides'

    slide_id = Column(Integer, primary_key = True)
    slide_link = Column(String, unique = True)
    presentation_id = Column(Integer, ForeignKey('google_presentations.presentation_id'))
    created_at = Column(DateTime)
    updated_at = Column(DateTime)

    presentation = relationship('Presentation', back_populates = 'slides')
    textboxes = relationship('Textbox', back_populates = 'slide')

    def __repr__(self):
        return f'<Slide slide_id:{self.slide_id} presentation_id:{self.presentation_id} link:{self.slide_link}>'


class Textbox(Base):
    __tablename__ = 'google_slide_textboxes'

    textbox_id = Column(Integer, primary_key = True)
    textbox_link = Column(String, unique = True)
    slide_id = Column(Integer, ForeignKey('google_slides.slide_id'))
    text = Column(String)
    created_at = Column(DateTime)
    updated_at = Column(DateTime)
    ts = Column(TSVECTOR)

    slide = relationship('Slide', back_populates = 'textboxes')

    def __repr__(self):
        return f'<Textbox textbox_id:{self.textbox_id} slide_id:{self.slide_id} link:{self.textbox_link}>'


import os.path
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
import json
from datetime import datetime

SCOPE = ['https://www.googleapis.com/auth/presentations.readonly']
PATH_TO_CREDENTIALS = 'API/token.json'  # The file token.json stores the user's access and refresh tokens
try:
    credentials = None
    if os.path.exists(PATH_TO_CREDENTIALS):
        credentials = Credentials.from_authorized_user_file(PATH_TO_CREDENTIALS, SCOPE)
    # If there are no (valid) credentials available, let the user log in.
    if not credentials or not credentials.valid:
        if credentials and credentials.expired and credentials.refresh_token:
            credentials.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file('API/credentials.json', SCOPE)
            credentials = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open(PATH_TO_CREDENTIALS, 'w') as token:
            token.write(credentials.to_json())
except Exception as e:
    raise ConnectionError('There is an issue with the credentials ' + str(e))


def get_presentation(presentation_id='19Fh8FULu6hNFVpb5iK6-NrBNqin3SLozE0emdJtstPM'):
    """Description: Calls the google slides API and returns the presentation based on the presentation ID
    Input:
        1) presentation_id : the google presentation id found on the url
    Returns:
        The presentation in json format
    """
    try:
        service = build('slides', 'v1', credentials=credentials)
        presentation = service.presentations().get(presentationId=presentation_id).execute()
        return presentation
    except Exception as e:
        raise ConnectionError('Cannot retrieve slides due to ' + str(e))

if __name__ == '__main__':
    get_presentation()



from datetime import datetime
from typing import Union

from pydantic import BaseModel


class PersonSerializer(BaseModel):
    id : int
    first_name : str
    last_name : str
    age : int
    gender : str
    nationality : str
    hasWatch : bool

    class Config:
        orm_mode = True

class PresentationSerializer(BaseModel):
    presentation_id : int
    presentation_link : str
    club_name : str
    revision_id : str
    created_at : datetime
    updated_at : datetime

    class Config:
        orm_mode = True

class SlideSerializer(BaseModel):
    slide_id : int
    slide_link : str
    presentation_id :int
    created_at : datetime
    updated_at : datetime

    class Config:
        orm_mode = True

class TextboxSerializer(BaseModel):
    textbox_id : int
    textbox_link : str
    slide_id : int
    text : str
    created_at : datetime
    updated_at : datetime

    class Config:
        orm_mode = True



from biosearch.DataPlatform.API import google_slides_api as google_api
import biosearch.DataPlatform.DB.database_ops as db_api
from datetime import datetime
import preprocessing_functions as ps
import json
import os
from pathlib import Path

def write_dictionary_to_csv(full_file_path, dictionary_data,presentation_id):
    data_list = []
    for slide_id, text_items in dictionary_data.items():
        record = str(presentation_id) + ';' + slide_id
        for text_id, text in text_items.items():
            data_list.append(record + ';' + text_id + ';' + text)
    with open(full_file_path, 'w') as f:
        for line in data_list:
            f.write(f"{line}\n")

def save_data_to_json_file(full_path, data):
    try:
        with open(full_path, 'w',
                  encoding='utf-8') as file:
            json.dump(data, file, ensure_ascii=False, indent=4)
    except Exception as e:
        print('Could not write file due to issue: ' + str(e))

def is_presentation_updated(db_presentation, api_presentation) -> bool:
    if db_presentation.get('revision_id') != api_presentation.get('revisionId'):
        return True
    return False
#'M5y2ytuyRmLSgQ'
def download_new_presentation_data(db_presentation):
    api_presentation = google_api.get_presentation(db_presentation.get('presentation_link'))
    if is_presentation_updated(db_presentation, api_presentation):
        db_api.update_google_presentation_revision_id(api_presentation.get('revisionId'))
        save_data_to_json_file(full_path= 'Data/raw_presentation_data_' + datetime.today().strftime('%Y-%m-%d_%H:%M:%S') + '.json',
                          data= api_presentation)

def preprocess_data(path_to_raw_data):
    return ps.preprocess_raw_file(path_to_raw_data)

def upload_file_to_temp_table(file_path):
    db_api.upload_file_to_temp_table(file_path)

def main():
    db_google_presentations = db_api.get_all_google_presentations()
    for db_presentation in db_google_presentations:
        download_new_presentation_data(db_presentation)
        preprocessed_slides = preprocess_data(sorted(Path('Data/').iterdir(), key=os.path.getmtime)[-1])
        write_dictionary_to_csv('data.csv', preprocessed_slides, 1)
        upload_file_to_temp_table('/Users/georgiosgkekas/ilabProjects/BioSearch/DataPlatform/data.csv')

if __name__ == '__main__':
    main()

